!
!     2٤κʬˡѤ
!     ٥ʡήĹΨη׻(rigid-rigid, T-T)
!
program SolveEigenvalueProblemOfZBenardConvection
  
  implicit none
  
  integer, parameter :: m = 41
  
  real(8) al, R, Pr, y(0:m)
  complex(8) ca(1:2*m-2, 1:2*m-2), cb(1:2*m-2, 1:2*m-2)
  complex(8) eigenvector(1:2*m-2), eigenvalue, W(0:m), T(0:m)
  
  call set_parameter(Pr, al, R)
  call calculate_matrices(m, al, R, Pr, ca, cb)
  call solve_eigenvalue_problem(2*m-2, ca, cb, eigenvector, eigenvalue)

  print *, "# wavenumber, Ra, real part of eigenvalue, imaginary part of eigenvalue"
  print *, "# ", al, R, dble(eigenvalue), dimag(eigenvalue)

  call set_axis(m, y)
  call calculate_eigenmode(m, y, eigenvector, W, T)
  
  stop
end program SolveEigenvalueProblemOfZBenardConvection

!
! ʻyɸꤹ롣
!
! y(i): iܤΥǥåγʻyɸ
!
! ɸֳֳʻҡ
!
subroutine set_axis(m, y)

  implicit none

  integer, intent(in)  :: m
  real(8), intent(out) :: y(0:m+1)  ! ʻyɸ

  integer j
  real(8) dtheta

  dtheta = 1.0d0/dble(m)
  do j=0,m
     y(j) = dble(j)*dtheta
  end do

end subroutine set_axis


! 
! ѥ᡼
! 
subroutine set_parameter(Pr, al, R)

  implicit none

  real(8), intent(out) :: Pr, al, R

!  write(*,*) 'input Prandtl number ?'; read(*,*) Pr
!  write(*,*) 'input alpha ?'; read(*,*) al
!  write(*,*) 'input Rayleigh number ?'; read(*,*) R
  Pr = 1.0d0
!  al = 3.1167d0
!  R = 1707.7600727796093d0
  al = 3.117d0
  R = 1707.762d0
end subroutine set_parameter


! 
! ̲ͭ $A x = s B x$ ˸ A, B׻
! 
!  Ǥ줾Υ֥åϤۤгʬΤߤιꤹ롣
!  ( Pr*(D^2-a^2)^2  -a^2*R*Pr)(W) = s*(D^2-a^2   0)(W)
!  ( 1               D^2-a^2  )(T)     (0         1)(T)
!
subroutine calculate_matrices(m, al, R, Pr, ca, cb)
  
  implicit none

  integer, intent(in) :: m
  real(8), intent(in) :: al, R, Pr
  complex(8), intent(out) :: ca(1:2*m-2, 1:2*m-2), cb(1:2*m-2, 1:2*m-2)

  integer i
  real(8) dz

  dz = 1.0d0/dble(m)

  ca(1:2*m-2, 1:2*m-2) = 0.0d0
  cb(1:2*m-2, 1:2*m-2) = 0.0d0

  i=1
  ca(i,i)   = ca(i,i) + Pr*1.0d0/dz**4
  ca(i,i)   = ca(i,i) + Pr*(6.0d0/dz**4+4.0d0*al**2/dz**2+al**4)
  ca(i+1,i) = ca(i+1,i) + Pr*(-4.0d0/dz**4-2.0d0*al**2/dz**2)
  ca(i+2,i) = ca(i+2,i) + Pr*1.0d0/dz**4

  ca(i,i+m-1) = ca(i,i+m-1) + 1.0d0

  ca(i+m-1,i) = ca(i+m-1,i) -al**2*R*Pr

  ca(i+m-1,i+m-1) = ca(i+m-1,i+m-1) -2.0d0/dz**2-al**2
  ca(i+1+m-1,i+m-1) = ca(i+1+m-1,i+m-1) + 1.0d0/dz**2

  cb(i,i) = cb(i,i) -2.0d0/dz**2 -al**2
  cb(i+1,i) = cb(i+1,i) + 1.0d0/dz**2

  cb(i+m-1,i) = cb(i+m-1,i) + 0.0d0

  cb(i,i+m-1) = cb(i,i+m-1) + 0.0d0

  cb(i+m-1,i+m-1) = cb(i+m-1,i+m-1) + 1.0d0

  i=2
  ca(i-1,i) = ca(i-1,i) + Pr*(-4.0d0/dz**4-2.0d0*al**2/dz**2)
  ca(i,i)   = ca(i,i) + Pr*(6.0d0/dz**4+4.0d0*al**2/dz**2+al**4)
  ca(i+1,i) = ca(i+1,i) + Pr*(-4.0d0/dz**4-2.0d0*al**2/dz**2)
  ca(i+2,i) = ca(i+2,i) + Pr*1.0d0/dz**4

  ca(i,i+m-1) = ca(i,i+m-1) + 1.0d0

  ca(i+m-1,i) = ca(i+m-1,i) -al**2*R*Pr

  ca(i-1+m-1,i+m-1) = ca(i-1+m-1,i+m-1) + 1.0d0/dz**2
  ca(i+m-1,i+m-1) = ca(i+m-1,i+m-1) -2.0d0/dz**2-al**2
  ca(i+1+m-1,i+m-1) = ca(i+1+m-1,i+m-1) + 1.0d0/dz**2

  cb(i-1,i) = cb(i-1,i) + 1.0d0/dz**2
  cb(i,i) = cb(i,i) -2.0d0/dz**2 -al**2
  cb(i+1,i) = cb(i+1,i) + 1.0d0/dz**2

  cb(i+m-1,i) = cb(i+m-1,i) + 0.0d0

  cb(i,i+m-1) = cb(i,i+m-1) + 0.0d0

  cb(i+m-1,i+m-1) = cb(i+m-1,i+m-1) + 1.0d0

  do i=3,m-3
    ca(i-2,i) = ca(i-2,i) + Pr*1.0d0/dz**4
    ca(i-1,i) = ca(i-1,i) + Pr*(-4.0d0/dz**4-2.0d0*al**2/dz**2)
    ca(i,i)   = ca(i,i) + Pr*(6.0d0/dz**4+4.0d0*al**2/dz**2+al**4)
    ca(i+1,i) = ca(i+1,i) + Pr*(-4.0d0/dz**4-2.0d0*al**2/dz**2)
    ca(i+2,i) = ca(i+2,i) + Pr*1.0d0/dz**4

    ca(i,i+m-1) = ca(i,i+m-1) + 1.0d0

    ca(i+m-1,i) = ca(i+m-1,i) -al**2*R*Pr

    ca(i-1+m-1,i+m-1) = ca(i-1+m-1,i+m-1) + 1.0d0/dz**2
    ca(i+m-1,i+m-1) = ca(i+m-1,i+m-1) -2.0d0/dz**2-al**2
    ca(i+1+m-1,i+m-1) = ca(i+1+m-1,i+m-1) + 1.0d0/dz**2

    cb(i-1,i) = cb(i-1,i) + 1.0d0/dz**2
    cb(i,i) = cb(i,i) -2.0d0/dz**2 -al**2
    cb(i+1,i) = cb(i+1,i) + 1.0d0/dz**2

    cb(i+m-1,i) = cb(i+m-1,i) + 0.0d0

    cb(i,i+m-1) = cb(i,i+m-1) + 0.0d0

    cb(i+m-1,i+m-1) = cb(i+m-1,i+m-1) + 1.0d0
  end do
  i=m-2
  ca(i-2,i) = ca(i-2,i) + Pr*1.0d0/dz**4
  ca(i-1,i) = ca(i-1,i) + Pr*(-4.0d0/dz**4-2.0d0*al**2/dz**2)
  ca(i,i)   = ca(i,i) + Pr*(6.0d0/dz**4+4.0d0*al**2/dz**2+al**4)
  ca(i+1,i) = ca(i+1,i) + Pr*(-4.0d0/dz**4-2.0d0*al**2/dz**2)

  ca(i,i+m-1) = ca(i,i+m-1) + 1.0d0

  ca(i+m-1,i) = ca(i+m-1,i) -al**2*R*Pr

  ca(i-1+m-1,i+m-1) = ca(i-1+m-1,i+m-1) + 1.0d0/dz**2
  ca(i+m-1,i+m-1) = ca(i+m-1,i+m-1) -2.0d0/dz**2-al**2
  ca(i+1+m-1,i+m-1) = ca(i+1+m-1,i+m-1) + 1.0d0/dz**2

  cb(i-1,i) = cb(i-1,i) + 1.0d0/dz**2
  cb(i,i) = cb(i,i) -2.0d0/dz**2 -al**2
  cb(i+1,i) = cb(i+1,i) + 1.0d0/dz**2

  cb(i+m-1,i) = cb(i+m-1,i) + 0.0d0

  cb(i,i+m-1) = cb(i,i+m-1) + 0.0d0

  cb(i+m-1,i+m-1) = cb(i+m-1,i+m-1) + 1.0d0

  i=m-1
  ca(i-2,i) = ca(i-2,i) + Pr*1.0d0/dz**4
  ca(i-1,i) = ca(i-1,i) + Pr*(-4.0d0/dz**4-2.0d0*al**2/dz**2)
  ca(i,i)   = ca(i,i) + Pr*(6.0d0/dz**4+4.0d0*al**2/dz**2+al**4)
  ca(i,i) = ca(i,i) + Pr*1.0d0/dz**4

  ca(i,i+m-1) = ca(i,i+m-1) + 1.0d0

  ca(i+m-1,i) = ca(i+m-1,i) -al**2*R*Pr

  ca(i-1+m-1,i+m-1) = ca(i-1+m-1,i+m-1) + 1.0d0/dz**2
  ca(i+m-1,i+m-1) = ca(i+m-1,i+m-1) -2.0d0/dz**2-al**2

  cb(i-1,i) = cb(i-1,i) + 1.0d0/dz**2
  cb(i,i) = cb(i,i) -2.0d0/dz**2 -al**2

  cb(i+m-1,i) = cb(i+m-1,i) + 0.0d0

  cb(i,i+m-1) = cb(i,i+m-1) + 0.0d0

  cb(i+m-1,i+m-1) = cb(i+m-1,i+m-1) + 1.0d0

end subroutine calculate_matrices


!
! ޤäͭ٥ȥ뤫, ήؿ԰⡼/Ǿ⡼ɤ
! 
subroutine calculate_eigenmode(m, y, eigenvector, W, T)

  implicit none

  integer, intent(in)  :: m
  real(8), intent(in)  :: y(0:m)
  complex(8), intent(in) :: eigenvector(1:2*m-2)
  complex(8), intent(out) :: W(0:m), T(0:m)

  integer j

  ! ޤ᤿yɸ y0 
  
  W(0) = 0.0d0   ! ꥼ
  W(m) = 0.0d0 ! Ʊ
  T(0) = 0.0d0   ! ꥼ
  T(m) = 0.0d0 ! Ʊ
  
  W(1:m-1) = eigenvector(1:m-1)
  T(1:m-1) = eigenvector(m:2*m-2)
!  W(0:m) = W(0:m)/W(m/2) ! y=1/2Ǥʣǿǵʲ
!  T(0:m) = T(0:m)/T(m/2) ! y=1/2Ǥʣǿǵʲ
  
  
  print *, "# eigenvector"
  print *, "# y(j), dble(W(j)), dimag(W(j)), dble(T(j)), dimag(T(j))"
  do j=0,m
     print *, y(j), dble(W(j)), dimag(W(j)), dble(T(j)), dimag(T(j))
  end do

end subroutine calculate_eigenmode


! 
! ͭ, ԰⡼ɤޤϺǾ⡼ɤ, б
! ͭؿ
! 
! Note:
!    оιΰ̲ͭLapackΥ֥롼 zgegv Ѥ.
!
!    zgegvλȤ, 
!    call zgegv( 'N', 'V', N, A, LDA, B, LDB, ALPHA, BETA,
!                VL, LDVL, VR, LDVR, WORK, LWORK, RWORK, INFO )
!
!    LDA, LDB leading dimension of A, B
!    ͭ lambda  lambda = alpha/beta η֤.
!    VLϺͭ٥ȥ, VRϱͭ٥ȥ
!    ܤȤ http://www.netlib.org/lapack/complex16/zgegv.f 򻲾Ȥ뤳.
!
subroutine solve_eigenvalue_problem(n, ca, cb, eigenvector, eigenvalue)

  implicit none

  integer, intent(in) :: n
  complex(8), intent(inout) :: ca(1:n, 1:n), cb(1:n, 1:n)
  complex(8), intent(out) :: eigenvector(1:n), eigenvalue

  integer info
  complex(8) alpha(1:n), beta(1:n), dummy(1:1, 1:n), rev(1:n, 1:n), work(1:24*n)
  real(8) work2(1:8*n)
  
  complex(8) lambda(1:n)
  integer i_max

  call zgegv( 'N', 'V', n, ca, n, cb, n, alpha, beta, &
       &      dummy, 1, rev, n, work, 24*n, work2, info )
  if( info /= 0 )then
     print *, 'error has occured in zgegv: error_code=', info
     call exit(1)
  end if
  lambda(1:n) = alpha(1:n)/beta(1:n)
  
  i_max = maxloc(dble(lambda), 1)

  eigenvector(1:n) = rev(1:n, i_max)
  eigenvalue = lambda(i_max)

end subroutine solve_eigenvalue_problem
